#!/usr/bin/env python
# encoding: utf-8

from django.contrib import admin
from django import forms
from django_hierarchy.forms import HierarchicalChoiceField

class HierarchicalAdminForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		super(HierarchicalAdminForm, self).__init__(*args, **kwargs)
		self.fields['parent'] = HierarchicalChoiceField(
			queryset = self.fields['parent'].queryset,
			required = False
		)
		
	def clean_parent(self):
		parent = self.cleaned_data['parent']
		if not self.instance is None and not parent is None:
			if self.instance.pk == parent.pk:
				raise forms.ValidationError(
					'%s cannot be a child of itself' % self.instance._meta.verbose_name.capitalize()
				)
				
		return parent

class HierarchicalAdmin(admin.ModelAdmin):
	list_display = (
		'hierarchical_name',
		'slug',
		'parent'
	)
	
	search_fields = ('name',)
	
	prepopulated_fields = {
		'slug': ('name',)
	}
	
	form = HierarchicalAdminForm
	
	def hierarchical_name(self, obj):
		parents = '&nbsp;&nbsp;&nbsp;&nbsp;' * obj.slug_hierarchical.count('/')
		return '%s%s' % (parents, obj.name)
	hierarchical_name.allow_tags = True
	hierarchical_name.short_description = 'Name'

class SortableHierarchicalAdmin(HierarchicalAdmin):
	list_display = HierarchicalAdmin.list_display + ('order',)
	search_fields = ('name',)


#!/usr/bin/env python
# encoding: utf-8

from django.forms import ChoiceField, Select, ValidationError

class HierarchicalSelect(Select):
	jquery_namespace = 'django.jQuery'
	
	def __init__(self, *args, **kwargs):
		self.jquery_namespace = kwargs.pop('jquery_namespace', self.jquery_namespace)
		super(HierarchicalSelect, self).__init__(*args, **kwargs)
	
	def render(self, name, value, attrs = None, choices = ()):
		from django.utils.safestring import mark_safe
		from django.forms.util import flatatt
		
		if value is None:
			value = ''
		
		final_attrs = self.build_attrs(attrs, name = name, value = value)
		output = [u'<input type="hidden" %s />' % flatatt(final_attrs)]
		
		def choicedict(choices, parent = None):
			cd = {}
			
			for (i, n, c) in choices:
				d = {
					'n': n,
					'c': choicedict(c, i),
				}
				
				if parent:
					d['p'] = parent
				
				cd[i] = d
			
			return cd
		
		from django.utils import simplejson
		choice_dict = choicedict(self.choices)
		
		def render_select(values):
			output = []
			index = 0
			choices = self.choices
			cd = choice_dict
			parent_value = 0
			
			for value in values:
				output.append(u'<select id="%s_%d">' % (name, index))
				output.append(u'<option value="%d">---</option>' % parent_value)
				options = self.render_options(choices, [value], max_depth = 1)
				
				if options:
					output.append(options)
				
				output.append('</select>');
				
				for (v, n, c) in choices:
					if int(v) == int(value):
						choices = c
				
				output.append('<script type="text/javascript">')
				output.append('%s(document).ready(function() {' % self.jquery_namespace)
				output.append(
					'prepare_%(name)s_options(%(index)d, %(dump)s);' % {
						'name': name,
						'index':  index,
						'dump': simplejson.dumps(cd)
					}
				)
				
				output.append('});</script>')
				index += 1
				parent_value = value
				cd = cd.get(value, {'c': {}}).get('c')
			
			return ''.join(output)
		
		if value and value != '':
			def parentage(value, choices):
				ps = []
				
				for k, v in choices.items():
					if int(k) == int(value):
						p = v.get('p')
						
						if not p is None:
							ps.append(p)
							ps.extend(
								parentage(p, choice_dict)
							)
					else:
						ps.extend(
							parentage(value, v['c'])
						)
				
				return ps
			
			values = parentage(value, choice_dict)
			values.insert(0, int(value))
			values.reverse()
			
			def childcount(value, choices):
				for k, v in choices.items():
					if int(k) == int(value):
						return len(v['c'])
					else:
						cc = childcount(value, v['c'])
						if cc > 0:
							return cc
							
				return 0
			
			if childcount(value, choice_dict) > 0:
				values.append(0)
			
			output.append(
				render_select(values)
			)
		else:
			output.append(
				render_select([0])
			)
			
			values = []
		
		output.append(
			'<script type="text/javascript">var %s_options = %s;' % (
				name, simplejson.dumps(choice_dict)
			)
		)
		
		output.append('var %s_field_index = %d;' % (name, len(values) - 1))
		output.append('var %s_selects = {' % name)
		
		for i, value in enumerate(values):
			output.append(
				'"%(id)d": "%(name)s_%(id)d"' % {
					'id': i,
					'name': name
				}
			)
			
			if i + 1 < len(values):
				output.append(',')
		
		output.append('};</script>')
		output.append('<script type="text/javascript">')
		output.append('\tfunction clear_%s_subselects(index) {' % name)
		output.append('\t\tfor(i in %s_selects) {' % name)
		output.append('\t\t\tif(i > index) {')
		output.append('\t\t\t\tid = %s_selects[i];' % name)
		output.append('\t\t\t\t%s("#" + id).remove();' % self.jquery_namespace)
		output.append('\t\t\t\t%s_selects[i] = null;' % name)
		output.append('\t\t\t}')
		output.append('\t\t}')
		output.append('\t\t%s_field_index = index;' % name)
		output.append('\t}')
		
		output.append('\tfunction prepare_%s_options(index, choices) {' % name)
		output.append(
			'\t\t%s("#%s_" + index).bind("change", function(e) {' % (
				self.jquery_namespace, name
			)
		)
		
		output.append(
			'\t\t%(jq)s("#id_%(name)s").val(%(jq)s(this).val())' % {
				'name': name,
				'jq': self.jquery_namespace
			}
		)
		
		output.append('\t\tclear_%s_subselects(index);' % name)
		output.append('\t\tvar id = %s(this).val();' % self.jquery_namespace)
		output.append('\t\tif(choices[id] !== undefined) {')
		output.append('\t\t\tsubchoices = choices[id]["c"];')
		
		output.append('\t\t\tarray_count = 0;')
		output.append('\t\t\tfor(i in subchoices) { array_count ++; }')
		output.append('\t\t\t\tif(array_count == 0) { return; }')
		
		output.append('\t\t\t\t\tvar select_id = "%s_" + (index + 1);' % name)
		output.append(
			'\t\t\t\t\tif(%(name)s_selects[index + 1]) {' % {
				'name': name
			}
		)
		
		output.append('\t\t\t\t\t} else { ')
		output.append(
			'\t\t\t\t\t\t%s(this).after("<select id=" + select_id + "></select>");' % \
				self.jquery_namespace
		)
		
		output.append(
			'\t\t\t\t\t\t%(name)s_selects[index + 1] = select_id' % {
				'name': name
			}
		)
		
		output.append('\t\t\t\t\t}')
		
		output.append('\t\t\t\t\toptions_html = "<option value=" + id + ">---</option>";')
		output.append('\t\t\t\t\tfor(i in subchoices) {')
		output.append('\t\t\t\t\t\tchoice = subchoices[i];')
		output.append('\t\t\t\t\t\toptions_html += "<option value=" + i + \
			">" + choice["n"] + "</option>";')
		output.append('\t\t\t\t\t}')
		output.append('\t\t\t\t\t%s("#" + select_id).html(options_html)' % \
			self.jquery_namespace)
		output.append(
			'\t\t\t\t\tprepare_%s_options(index + 1, subchoices);' % name
		)
		output.append('\t\t\t\t} else {')
		output.append('\t\t\t\t\tclear_%s_subselects(index);' % name)
		output.append('\t\t\t\t}')
		output.append('\t\t\t}')
		output.append('\t\t);')
		
		output.append('\t\t%s_field_index = index;' % name)
		output.append('\t}')
		output.append('</script>')
		
		return mark_safe(u'\n'.join(output))
	
	def render_options(self, choices, selected_choices, max_depth = -1):
		from django.utils.encoding import StrAndUnicode, force_unicode
		from django.utils.html import escape, conditional_escape
		from itertools import chain
		
		def render_option(option_value, option_label, children, depth = 0):
			option_value = force_unicode(option_value)
			selected_html = (option_value in selected_choices) and \
				u' selected="selected"' or ''
			items = []
			
			items.append(
				u'<option value="%s"%s>%s%s</option>' % (
				escape(option_value), selected_html,
				'&nbsp;' * depth * 3,
				conditional_escape(force_unicode(option_label)))
			)
			
			if depth + 1 < max_depth or max_depth == -1:
				for (child_option, child_label, child_children) in children:
					items.append(
						render_option(child_option, child_label, child_children, depth + 1)
					)
				
			return ''.join(items)
		
		# Normalize to strings.
		selected_choices = set([force_unicode(v) for v in selected_choices])
		output = []
		
		for option_value, option_label, children in choices:
			output.append(
				render_option(option_value, option_label, children)
			)
		
		return u'\n'.join(output)

class HierarchicalChoiceField(ChoiceField):
	widget = HierarchicalSelect
	
	def __init__(self, queryset, *args, **kwargs):
		super(HierarchicalChoiceField, self).__init__(*args, **kwargs)
		self.queryset = queryset
		self.choices = queryset.get_tree()
	
	def valid_value(self, value):
		return isinstance(value, self.queryset.model)
	
	def to_python(self, value):
		from django.core.validators import EMPTY_VALUES
		
		print value
		if value in EMPTY_VALUES:
			return None
		
		try:
			value = self.queryset.get(pk = value)
		except self.queryset.model.DoesNotExist:
			raise ValidationError(self.error_messages['invalid_choice'] % {'value': value})
		
		return value


#!/usr/bin/env python
# encoding: utf-8

from django.db.models import Manager

class HierarchicalManager(Manager):
	def get_tree(self):
		return self.get_query_set().get_tree()
	
	def get_query_set(self):
		return self.model.QuerySet(self.model)


#!/usr/bin/env python
# encoding: utf-8

from django.db import models
from django_hierarchy.managers import HierarchicalManager

class Hierarchical(models.Model):
	name = models.CharField(max_length = 50)
	slug = models.SlugField(max_length = 20)
	slug_hierarchical = models.CharField(max_length = 255, editable = False, unique = True)
	parent = models.ForeignKey('self', related_name = 'children', null = True)
	objects = HierarchicalManager()
	
	@property
	def ancestor_count(self):
		return self.slug_hierarchical.count('/')
	
	def __unicode__(self):
		return self.name
	
	def hierarchy_changed(self, old):
		return old.slug_hierarchical != self.slug_hierarchical
	
	def save(self, *args, **kwargs):
		if self.parent:
			self.slug_hierarchical = '%s/%s' % (self.parent.slug_hierarchical, self.slug)
		else:
			self.slug_hierarchical = self.slug
		
		if not self.pk is None and self.children.count() > 0:
			old = type(self).objects.get(pk = self.pk)
			super(Hierarchical, self).save(*args, **kwargs)
			
			if self.hierarchy_changed(old):
				for child in self.children.all():
					child.save()
			
			return
		
		super(Hierarchical, self).save(*args, **kwargs)
	
	class QuerySet(models.query.QuerySet):
		def get_tree(self):
			from django_hierarchy import helpers
			return helpers.recurse(self.filter(parent__isnull = True))
	
	class Meta:
		abstract = True
		ordering = ('slug_hierarchical',)
		unique_together = ('parent', 'slug')
		
class SortableHierarchical(Hierarchical):
	order = models.PositiveSmallIntegerField(default = 0)
	order_hierarchical = models.CharField(max_length = 255, editable = False)
	
	def hierarchy_changed(self, old):
		changed = super(SortableHierarchical, self).hierarchy_changed(old)
		if not changed:
			return old.order_hierarchical != self.order_hierarchical
		
		return True
	
	def save(self, *args, **kwargs):
		if self.parent:
			self.order_hierarchical = '%s/%s' % (
				str(self.parent.order_hierarchical).zfill(3),
				str(self.order).zfill(3)
			)
		else:
			self.order_hierarchical = str(self.order).zfill(3)
		
		super(SortableHierarchical, self).save(*args, **kwargs)
	
	class Meta:
		abstract = True
		ordering = ('order_hierarchical',)
		unique_together = ('parent', 'slug')


#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
	name = 'django-hierarchical',
	version = '0.1.4c',
	description = 'Hierarchical models and form fields for Django',
	author = 'Moxy Park',
	author_email = 'moxypark@me.com',
	url = 'http://moxypark.co.uk/',
	packages = find_packages(),
	package_dir = {
		'django_hierarchy': 'django_hierarchy'
	},
	install_requires = [
		'django>=1.2.1'
	],
	classifiers = [
		'Development Status :: 3 - Alpha',
		'Environment :: Web Environment',
		'Framework :: Django',
		'Intended Audience :: Developers',
		# 'License :: OSI Approved :: BSD License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Framework :: Django',
    ]
)